package main


/*
pbmac-2[10:04:56]~/Desktop/Go>go test *.go
pbmac-2[10:05:52]~/Desktop/Go>go test -coverprofile cover.out *.go
pbmac-2[10:06:35]~/Desktop/Go>go tool cover -html=cover.out
*/
import (
  "bufio"
  "fmt"
  "os"
  "strings"
  "strconv"
)

type studente struct {
  nome string
  voti []int
}

func avg(voti []int) float64 {
  s := 0
  for _, voto := range voti {
    if voto == 32 {
      s += 30
    } else {
      s += voto
    }
  }
  return float64(s) / float64(len(voti))
}

func parseLine(riga string) (s studente, ok bool) {
  x := strings.Split(riga, ",")
  if len(x) < 2 {
    fmt.Fprintf(os.Stderr, "La riga <%s> contiene troppo pochi campi\n", riga)
    return
  }
  s.nome = x[0]
  if len(s.nome) == 0 {
    fmt.Fprintf(os.Stderr, "La riga <%s> contiene un nome vuoto\n", riga)
    return
  }
  for i := 1; i < len(x); i++ {
    voto, err := strconv.Atoi(x[i])
    if err != nil {
      fmt.Fprintf(os.Stderr, "La riga <%s> contiene un numero non convertibile (%s)\n", riga, x[i])
      return
    }
    if voto < 18 || voto > 30 && voto != 32 {
      fmt.Fprintf(os.Stderr, "La riga <%s> contiene un voto non valido (%d)\n", riga, voto)
      return
    }
    s.voti = append(s.voti, voto)
  }
  ok = true
  return
}

func parseFile(nomeFile string) (s []studente, ok bool) {
  f, err := os.Open(nomeFile)
  defer f.Close()
  if err != nil {
    fmt.Fprintf(os.Stderr, "Errore nell'apertura del file %s\n", nomeFile)
    return
  }
  scanner := bufio.NewScanner(f)
  for scanner.Scan() {
    riga := scanner.Text()
    studente, ok := parseLine(riga)
    if !ok {
      fmt.Fprintf(os.Stderr, "Ignoro la riga e vado avanti\n")
      continue
    }
    s = append(s, studente)
  }
  ok = true
  return
}

func convert110(x float64) float64 {
  return 110 * x / 30
}

func writeOut(s []studente, nomeFile string) (ok bool) {
  f, err := os.Create(nomeFile)
  defer f.Close()
  if err != nil {
    fmt.Fprintf(os.Stderr, "Errore nella creazione del file %s\n", nomeFile)
    return
  }
  for _, studente := range s {
    fmt.Fprintf(f, "%-30s\t%7.2f\n",
      studente.nome,
      convert110(avg(studente.voti)))
  }
  ok = true
  return
}

func main() {
  fileDiInput := os.Args[1]
  fileDiOutput := os.Args[2]
  s, ok := parseFile(fileDiInput)
  if !ok {
    os.Exit(1)
  }
  ok = writeOut(s, fileDiOutput)
  if !ok {
    os.Exit(1)
  }
}
